
package Docclient::Config;
%Docclient::Config::Config =
	(
	'ServerClass'	=>	'Docserver::Srv',
	'ChunkSize'	=>	512 * 32,
	'server'	=>	'agape',
	'port'		=>	5454,
	'application'	=>	'Docserver',
	'in_format'	=>	'doc',
	'out_format'	=>	'txt',
	);

